<?php

namespace FiloBlu\ExtInventory\Model\ShippingAlgorithms\SourceSlots;

use Magento\Framework\App\ObjectManager;
use Magento\InventoryApi\Api\Data\SourceInterface;
use Magento\Store\Model\ScopeInterface;

use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\Message\ManagerInterface;
use Magento\Inventory\Model\ResourceModel\Source\Collection as SourceCollection;
use Magento\Framework\App\Config\ScopeConfigInterface;

use Magento\InventoryApi\Api\GetSourcesAssignedToStockOrderedByPriorityInterface;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;

use Magento\InventoryApi\Api\GetSourceItemsBySkuInterface;

class SourcesData {

    const DEFAULT_SOURCE_CODE_FALLBACK = 'magento_snatt';

    protected $_serializer;
    protected $_messageManager;
    protected $sourcesCollection;
    protected $scopeConfig;
    protected $orderCollectionFactory;
    protected $getSourcesAssignedToStockOrderedByPriority;
    protected $getSourceItemsBySkuInterface;

    public function __construct(
        SerializerInterface $serializer,
        ManagerInterface $messageManager,
        SourceCollection $getSourcesCollection,
        ScopeConfigInterface $scopeConfig,
        CollectionFactory $collectionFactoryInterface,
        GetSourcesAssignedToStockOrderedByPriorityInterface $getSourcesAssignedToStockOrderedByPriority,
        GetSourceItemsBySkuInterface $getSourceItemsBySkuInterface
    ) {
        $this->_serializer = $serializer;
        $this->_messageManager = $messageManager;
        $this->sourcesCollection = $getSourcesCollection;
        $this->scopeConfig = $scopeConfig;
        $this->orderCollectionFactory = $collectionFactoryInterface;
        $this->getSourcesAssignedToStockOrderedByPriority = $getSourcesAssignedToStockOrderedByPriority;
        $this->getSourceItemsBySkuInterface = $getSourceItemsBySkuInterface;
    }

    /**
     * @return array
     * Return array of source codes and max daily slots
     */
    public function getSourcesDailyMaxSlots(){
        // Get the daily maximum slots configuration from the backend
        $config = $this->getSourceExtraConfig();

        // Prettify the result with array key => values
        $result = array();
        foreach ($config as $key => &$config_data) {
            if ($config_data['source'] == $this->getMainSourceCode()){
                $config_data['daily_max_slots'] = 1000000;
            }
            $result[$config_data['source']]['max'] = $config_data['daily_max_slots'];
        }
        return $result;
    }

    public function getSourceCodeFromThirdPartiesCode($code){
        $config = $this->getSourceExtraConfig();
        foreach ($config as $key => &$config_data) {
            if ($config_data['source'] == $code){
                return $config_data['source'];
            }
        }
        return false;
    }

    public function getSourceExtraConfig(){
        $config = $this->scopeConfig->getValue('msi/sources/extra_configs', ScopeInterface::SCOPE_STORE);
        if (!$config){
            $this->_messageManager->addErrorMessage("Missing system configurations for FiloBlu MSI module, they are mandatory for the selected shipping algorithm!");
        }
        $config = $this->_serializer->unserialize($config);
        $config = array_values($config);
        return $config;
    }

    /**
     * @return string
     * Return array of source_code of the main inventory source
     */
    public function getMainSourceCode(){
        $code = $this->scopeConfig->getValue('msi/sources/main_source_code', ScopeInterface::SCOPE_STORE);
        if(!$code){
            $code = self::DEFAULT_SOURCE_CODE_FALLBACK;
        }
        return $code;
    }

    public function getOrderOccupiedSlotStatuses(){
        $statuses = $this->scopeConfig->getValue('msi/sources/occupied_slot_statuses', ScopeInterface::SCOPE_STORE);
        return $statuses;
    }

    /**
     * @return SourceCollection
     * Return a collection of sources ordered by priority DESC and with $stock_id filter
     */
    public function getSourcesListCollection($stock_id = null,$return_array = false){
//array('inventory_source_stock_link' => 'inventory_source_stock_link')
        $this->sourcesCollection->getSelect()
            ->joinLeft('inventory_source_stock_link',
                'main_table.source_code=inventory_source_stock_link.source_code',
                array('priority' => 'inventory_source_stock_link.priority',
                    'stock_id' => 'inventory_source_stock_link.stock_id',
                    'link_id' => 'inventory_source_stock_link.link_id'
                )
            );

        if ($stock_id){
            $this->sourcesCollection->getSelect()->where("stock_id=".$stock_id);
        }

        $this->sourcesCollection->setOrder('priority','ASC');

        //$this->sourcesCollection = $this->sourcesCollection->load();
        return $this->sourcesCollection;
    }

    public function getSourcesListCodeArray($stock_id,$by_id = false){
        $sourcesCollection = $this->getSourcesListCollection($stock_id);
        $return = array();
        foreach ($sourcesCollection as $source) {
            if($by_id){
                $return[$source->getLinkId()] = $source->getSourceCode();
            } else {
                $return[] = $source->getSourceCode();
            }
        }
        return $return;
    }

    public function getSourcesArrayById($stock_id){
        return $this->getSourcesListCodeArray($stock_id,true);
    }

    /**
     * Get enabled sources ordered by priority by $stockId
     *
     * @param int $stockId
     * @return array or sourced codes (ordered by priority)
     *
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getEnabledSourcesOrderedByPriorityByStockId(int $stockId): array
    {
        try {
            $sources = $this->getSourcesAssignedToStockOrderedByPriority->execute($stockId);
            $sources = array_filter($sources, function (SourceInterface $source) {
                return $source->isEnabled() && $source->getSourceCode() != 'default';
            });
            return $sources;
        } catch (\Exception $e) {
            return array();
        }

    }

    public function getEnabledSourcesCodesArrayOrderedByPriorityByStockId(int $stockId): array
    {
        $result = array();
        $sources = $this->getEnabledSourcesOrderedByPriorityByStockId($stockId);
        foreach ($sources as $source) {
            $result[] = $source->getSourceCode();
        }
        return $result;
    }

    public function getSourcesOccupiedSlotsNumber(){

        $collection = $this->orderCollectionFactory->create();
        $main_source_code = $this->getMainSourceCode();

        $collection->getSelect()
            ->joinLeft(array('SS'=>'sales_shipment'),
                'main_table.entity_id=SS.order_id',
                array('order_entity_id' => 'main_table.entity_id',
                    'order_increment' => 'main_table.increment_id',
                    'order_status' => 'main_table.status'
                )
            )
            ->joinLeft(array( 'ISS' => 'inventory_shipment_source'),
                'SS.entity_id=ISS.shipment_id',
                array('source_code' => 'ISS.source_code')
            );

        $collection->getSelect()->where("main_table.status IN ('pending','processing')");
        $collection->getSelect()->where("ISS.source_code IS NOT NULL");
        $collection->getSelect()->where("ISS.source_code <> '{$main_source_code}'");

        //$this->sourcesCollection = $this->sourcesCollection->load();
        $occupied_slots = array();
        if($collection->getSize() == 0){
            return $occupied_slots;
        }
        // Just counting the query result = occupied slots
        foreach ($collection as $slot) {

            if(isset($occupied_slots[$slot['source_code']]['occupied'])){
                $occupied_slots[$slot['source_code']]['occupied']++;
            } else {
                $occupied_slots[$slot['source_code']]['occupied'] = 1;
            }
        }

        return $occupied_slots;

        /*
        $sql = "SELECT main_table.entity_id AS 'order_entity_id'
                ,main_table.increment_id AS 'order_increment'
                ,main_table.status AS 'order_status'
                ,ISS.source_code
                FROM sales_order AS main_table
                LEFT JOIN sales_shipment SS ON SS.order_id = main_table.entity_id
                LEFT JOIN inventory_shipment_source ISS ON ISS.shipment_id = SS.entity_id
                WHERE main_table.status IN ('pending','processing')
                AND ISS.source_code IS NOT NULL
                AND ISS.source_code <> 'santoni_snatt'";
        */

    }


    public function getSourceItemsBySkuInterface($sku){
        $sourceItems = $this->getSourceItemsBySkuInterface->execute($sku);
        return $sourceItems;
    }

}